# -*- coding: utf-8 -*-

# Copyright 2013-2014 Jorge Estrada. 
#
# This file is part of azucar.cppUtils project. 
# It is subject to the license terms in the 
# LICENSE file found in the top-level directory of this distribution. 
# No part of azucar.cppUtils project, including this file, may be copied,
# modified, propagated, or distributed except according to the terms contained 
# in the LICENSE file.

import azucar.buildeng.CMDB as CMDB
from copy import copy
import os.path

from azucar.buildeng.CMDB import CMDB_ANY, combine

null_path = '/dev/null'

def setupPlatforms(database):

    templateTuple = CMDB.CMTuple(project=CMDB_ANY,
                                 role='fmn',
                                 stage=CMDB_ANY,
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
      
    dbview = CMDB.CMDBView(database, templateTuple)

    dbview.set('targetRepository', value="/home/jestrada/projects/artifactrepo")

    templateTuple = CMDB.CMTuple(project=CMDB_ANY,
                                 role='galacdata1',
                                 stage=CMDB_ANY,
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
      
    dbview = CMDB.CMDBView(database, templateTuple)

    dbview.set('targetRepository', value="/data/jestrada/artifactrepo")

def setupBuild(database):
    templateTuple = CMDB.CMTuple(project=CMDB_ANY,
                                 role=CMDB_ANY,
                                 stage=CMDB_ANY,
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
    
    dbview = CMDB.CMDBView(database, templateTuple)
    dbview.set('configBuildFile', 'configBuild.py')
    dbview.set('processors', 1)


    #### galacdata1 ###
    templateTuple = CMDB.CMTuple(project=CMDB_ANY,
                                 role='galacdata1',
                                 stage=CMDB_ANY,
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
    
    dbview = CMDB.CMDBView(database, templateTuple)
    dbview.set('processors', 4)
    
    
    templateTuple = CMDB.CMTuple(project=CMDB_ANY,
                                 role='galacdata1',
                                 stage='build',
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
    
    dbview = CMDB.CMDBView(database, templateTuple)
    
    dbview.set('DotPath', "/usr/bin/dot")
    dbview.set('CompilerFlags', '-Wall -g -pthread -DBOOST_IOSTREAMS_NO_LIB -DBOOST_TEST_DYN_LINK -frandom-seed="$TARGET"')
    dbview.set('LinkFlags', '-pthread -Wl,--as-needed')
    
    boostLibraryType = "-mt"
    dbview.set('BoostLibraryType', boostLibraryType)
    
    libraries = map(lambda x: x+boostLibraryType, \
                    ["boost_regex", "boost_filesystem", "boost_thread", "boost_date_time", 
                     "boost_program_options", "boost_system", "boost_unit_test_framework",
                     "boost_iostreams"])
    # Add support for internationalization
    libraries.append("icuuc")
    
    dbview.set('BoostLibraries', libraries)
    dbview.set('BoostLibdirs', [])
    dbview.set('BoostIncludes', [])
    dbview.set('BoostVersion', '1.44.0')
    
    libraries = map(lambda x: x+boostLibraryType, \
                            ["boost_python",])
    
    dbview.set('BoostPythonLibraries', libraries)
    
    dbview.set('XercesLibraries', [":libxerces-c.so.27"])
    dbview.set('XercesLibdirs', [])
    dbview.set('XercesIncludes', ["/usr/include/xercesc-2.7.0"])
    
    dbview.set('LokiLibraries', ["loki"])
    dbview.set('LokiLibdirs', [])
    dbview.set('LokiIncludes', [])
    
    dbview.set('PythonIncludes', ["/usr/include/python2.7",])
    dbview.set('PythonLibdirs', ["/usr/lib64/python-2.7",])
    dbview.set('PythonLibraries', ["python2.7", "util"])

    dbview.set('MysqlppLibraries', ["mysqlclient", "mysqlpp"])
    dbview.set('MysqlppLibdirs', ["/usr/lib64/mysql"])
    dbview.set('MysqlppIncludes', ["/usr/include/mysql++", "/usr/include/mysql"])

    dbview.set('LogLibraries', ["log4cpp"])
    dbview.set('LogLibdirs', ["/usr/local/log4cpp/lib"])
    dbview.set('LogIncludes', ["/usr/local/log4cpp/include"])
    
    dbview.set('GmpLibraries', ["gmp", "gmpxx"])
    dbview.set('GmpLibdirs', [])
    dbview.set('GmpIncludes', [])
    
    dbview.set('GslLibraries', ["gsl", "gslcblas"])
    dbview.set('GslLibdirs', [])
    dbview.set('GslIncludes', [])

    dbview.set('CgiccLibraries', ["cgicc"])
    dbview.set('CgiccLibdirs', ["/usr/local/cgicc/lib"])
    dbview.set('CgiccIncludes', ["/usr/local/cgicc/include"])
    
    dbview.set('VmimeLibraries', ["vmime"])
    dbview.set('VmimeLibdirs', ["/usr/local/libvmime-0.9.1-patched/lib"])
    dbview.set('VmimeIncludes', ["/usr/local/libvmime-0.9.1-patched/include"])
    
    

    #### fmn ###

    templateTuple = CMDB.CMTuple(project=CMDB_ANY,
                                 role='fmn',
                                 stage=CMDB_ANY,
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
    
    dbview = CMDB.CMDBView(database, templateTuple)
    dbview.set('processors', 2)
    
    templateTuple = CMDB.CMTuple(project=CMDB_ANY,
                                 role='fmn',
                                 stage='build',
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
    
    dbview = CMDB.CMDBView(database, templateTuple)
    dbview.set('DotPath', null_path)
    
    dbview.set('CompilerFlags', 
               '-Wall -g -pthread -DBOOST_IOSTREAMS_NO_LIB -DBOOST_TEST_DYN_LINK -frandom-seed="$TARGET"')
    dbview.set('LinkFlags', '-pthread')
    
    boostLibraryType = "-mt"
    
    dbview.set('BoostLibraryType', boostLibraryType)
    
    libraries = map(lambda x: x+boostLibraryType, \
                    ["boost_regex", "boost_filesystem", "boost_thread", "boost_date_time",
                     "boost_program_options", "boost_system", "boost_unit_test_framework",
                     "boost_iostreams",])
    # Add support for internationalization
    libraries.append("icuuc")
    
    dbview.set('BoostLibraries', libraries)
    dbview.set('BoostLibdirs', ["/usr/local/boost_1_44_0/lib", "/usr/local/icu4.4.2/lib"])
    dbview.set('BoostIncludes', ["/usr/local/boost_1_44_0/include",])
    dbview.set('BoostVersion', '1.44.0')
    
    libraries = map(lambda x: x+boostLibraryType, \
                            ["boost_python",])
    
    dbview.set('BoostPythonLibraries', libraries)
    
    dbview.set('XercesLibraries', ["xerces-c"])
    dbview.set('XercesLibdirs', ["/usr/local/xercesc/lib"])
    dbview.set('XercesIncludes', ["/usr/local/xercesc/include"])
    
    dbview.set('LokiLibraries', ["loki"])
    dbview.set('LokiLibdirs', ["/usr/local/loki/lib"])
    dbview.set('LokiIncludes', ["/usr/local/loki/include"])
    
    dbview.set('PythonIncludes', ["/usr/local/python-2.7/include/python2.7",])
    dbview.set('PythonLibdirs', ["/usr/local/python-2.7/lib",])
    dbview.set('PythonLibraries', ["python2.7", "util"])

    dbview.set('MysqlppLibraries', ["mysqlclient", "mysqlpp"])
    dbview.set('MysqlppLibdirs', ["/usr/local/mysql++-3.1.0/lib", "/usr/lib64/mysql"])
    dbview.set('MysqlppIncludes', ["/usr/local/mysql++-3.1.0/include",
                                   "/usr/include/mysql"])

    dbview.set('LogLibraries', ["log4cpp"])
    dbview.set('LogLibdirs', ["/usr/local/log4cpp/lib"])
    dbview.set('LogIncludes', ["/usr/local/log4cpp/include"])
    
    dbview.set('GmpLibraries', ["gmpxx", "gmp"])
    dbview.set('GmpLibdirs', [])
    dbview.set('GmpIncludes', [])
    
    dbview.set('GslLibraries', ["gsl", "gslcblas"])
    dbview.set('GslLibdirs', ["/usr/local/gsl1.14/lib"])
    dbview.set('GslIncludes', ["/usr/local/gsl1.14/include"])

    dbview.set('CgiccLibraries', ["cgicc"])
    dbview.set('CgiccLibdirs', ["/usr/local/cgicc-3.2.9/lib"])
    dbview.set('CgiccIncludes', ["/usr/local/cgicc-3.2.9/include"])
    
    dbview.set('VmimeLibraries', ["vmime"])
    dbview.set('VmimeLibdirs', ["/usr/local/libvmime-0.9.1-patched/lib"])
    dbview.set('VmimeIncludes', ["/usr/local/libvmime-0.9.1-patched/include"])
    

    
def setupCppUtils(database):
    templateTuple = CMDB.CMTuple(project='cppUtils',
                                 role='galacdata1',
                                 stage=CMDB_ANY,
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
    
    dbview = CMDB.CMDBView(database, templateTuple)

    dbview.set('installPath', '/data/jestrada/sharedlibraries')
    
    templateTuple = CMDB.CMTuple(project='cppUtils',
                                 role='fmn',
                                 stage=CMDB_ANY,
                                 version=CMDB_ANY,
                                 key=CMDB_ANY)
    
    dbview = CMDB.CMDBView(database, templateTuple)

    dbview.set('installPath', '/home/jestrada/projects/sharedlibraries')
    
def loadDatabase():
    database = CMDB.CMDB()
    
    setupBuild(database)
    
    setupPlatforms(database)
    
    setupCppUtils(database)
    

    return database
    
