/*! \file azucar/utils/exceptions.hpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#ifndef EXCEPTIONS_HPP_INCLUDED_
#define EXCEPTIONS_HPP_INCLUDED_

#include <string>

namespace azucar {
namespace utils {

/**
 * New exception class to signal processing errors
 */
class BaseException : public std::exception {
public:
  /**
   * Create a new BaseException with the associated message msg.
   * If an error is found during msg copying (i.e., a memory run out
   * problem), the exception will not work, and a memory related
   * exception will be thrown.
   */
  BaseException(const std::string &msg): message_(msg) {};

  const char *what() const throw() {
    return message_.c_str();
  }
  /**
   * Retrieve the message associated with this exception.
   */
  const std::string getMessage() {
    return message_;
  }

  virtual ~BaseException() throw () {};
private:
  std::string message_;
};

/*! \brief Shows an immediate need to finish the application,
 *  due to a failure on std streams.
 *
 * Do not write either to stdout nor read from stdin.
 *
 * \sa BaseException
 */
class StdStreamException : public BaseException {
public:
  /*! \sa BaseException::BaseException()
   */
  StdStreamException(const std::string &msg) : BaseException(msg) {};

  /*! \sa BaseException::~BaseException()
   */
  virtual ~StdStreamException() throw() {};
};

}
}


#endif // EXCEPTIONS_HPP_INCLUDED_
