/*! \file azucar/utils/extensibleI.hpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#ifndef EXTENSIBLEI_HPP_
#define EXTENSIBLEI_HPP_

#include <set>

#include <boost/weak_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>

class ExtensibleI;
class ExtensionMgr;
class PropertyObjectI;

typedef boost::shared_ptr<PropertyObjectI> PropertyObjectPtr;
typedef boost::weak_ptr<ExtensibleI> ExtensiblePtr;
typedef boost::shared_ptr<ExtensionMgr> ExtensionMgrPtr;

class ExtensionMgr {
private:
  std::set<PropertyObjectPtr> properties_;
public:
  void registerProperty(PropertyObjectPtr property);

  const std::set<PropertyObjectPtr> &getRegisteredProperties() const;
};

class PropertyObjectI {
public:
  virtual bool remove(ExtensiblePtr extensibleObject) = 0;
  virtual ~PropertyObjectI() {};
};

class ExtensibleI 
  : public boost::enable_shared_from_this<ExtensibleI> {
protected:
  ExtensionMgrPtr manager_;
  ExtensiblePtr thisWeakPtr_;
public:
  virtual void registerExtensionMgr(ExtensionMgrPtr manager);

  virtual ~ExtensibleI();
};

template<class ExtensibleClass> ExtensiblePtr
getExtensiblePtr(typename ExtensibleClass::ptr originalPtr) {
  return boost::dynamic_pointer_cast<ExtensibleI, ExtensibleClass>(originalPtr);
}

#endif /*EXTENSIBLEI_HPP_*/
