/*! \file azucar/utils/mail/mailCommon.hpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#ifndef MAILCOMMON_HPP_
#define MAILCOMMON_HPP_

#include <string>
#include <vector>

#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/convenience.hpp>

#include <boost/preprocessor/iteration/local.hpp>
#include <boost/preprocessor/cat.hpp>
#include <boost/preprocessor/tuple.hpp>
#include <boost/preprocessor/seq.hpp>
#include <boost/preprocessor/stringize.hpp>
#include <boost/preprocessor/punctuation.hpp>


#include <log4cpp/Category.hh>

class MailConfiguration {
private:
#define MAILCOMMON_var_seq ((mail_expeditor, std::string))\
((mail_sasl_user, std::string))\
((mail_sasl_passwd, std::string))\
((smtp_server_url, std::string))\
((use_smtp_auth, bool))

#define MYMACRO(data) static BOOST_PP_TUPLE_ELEM(2,1,data) \
BOOST_PP_CAT(BOOST_PP_TUPLE_ELEM(2,0,data), _);
#define BOOST_PP_LOCAL_MACRO(n) MYMACRO(BOOST_PP_SEQ_ELEM(n, MAILCOMMON_var_seq))
#define BOOST_PP_LOCAL_LIMITS (0, BOOST_PP_SEQ_SIZE(MAILCOMMON_var_seq)-1)
#include BOOST_PP_LOCAL_ITERATE()
#undef MYMACRO


public:
  static const std::string category_name;

  static log4cpp::Category &logger();

#define MYMACRO(data) static const BOOST_PP_TUPLE_ELEM(2,1,data) \
 &BOOST_PP_TUPLE_ELEM(2,0,data);
#define BOOST_PP_LOCAL_MACRO(n) MYMACRO(BOOST_PP_SEQ_ELEM(n, MAILCOMMON_var_seq))
#define BOOST_PP_LOCAL_LIMITS (0, BOOST_PP_SEQ_SIZE(MAILCOMMON_var_seq)-1)
#include BOOST_PP_LOCAL_ITERATE()
#undef MYMACRO


  /*!
   * \exception azucar::utils::BaseException If there is any problem.
   */
  static void read(const boost::filesystem::path &configFile);
};


#endif /*MAILCOMMON_HPP_*/
