/*! \file azucar/utils/mail/mailUtil.hpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#ifndef MAILUTIL_HPP_
#define MAILUTIL_HPP_

#include <string>

#include <boost/thread/mutex.hpp>
#include <boost/utility.hpp>

#include <vmime/vmime.hpp>

class MailSender : boost::noncopyable {
public:
  MailSender() {};
  ~MailSender() {};
  /*! \brief Lock mailing system to send an email.
   * 
   * It returns when the email has been sent, or an error
   * has occur (it throws the exception).
   * 
   * The lock is always freed once the function returns
   * (even in the case of a exception).
   */
  void sendMail(vmime::ref<vmime::message> msg);
  void sendMail(vmime::ref<vmime::message> msg, 
      const std::string &smtp_user, const std::string &smtp_key);
private:
  mutable boost::mutex m_mutex;
};

// SASL authentication handler
class interactiveAuthenticator : public vmime::security::sasl::defaultSASLAuthenticator
{
public:
  interactiveAuthenticator();
  interactiveAuthenticator(const std::string &user, const std::string &key); 
  const std::vector <vmime::ref <vmime::security::sasl::SASLMechanism> > getAcceptableMechanisms
    (const std::vector <vmime::ref <vmime::security::sasl::SASLMechanism> >& available,
     vmime::ref <vmime::security::sasl::SASLMechanism> suggested) const;

  void setSASLMechanism(vmime::ref <vmime::security::sasl::SASLMechanism> mech);

  const vmime::string getUsername() const;
  const vmime::string getPassword() const;
private:

  mutable vmime::string m_username;
  mutable vmime::string m_password;
};

#endif /*MAILUTIL_HPP_*/
