/*! \file azucar/utils/version.hpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#ifndef VERSION_HPP_
#define VERSION_HPP_

#include <string>

#define API_MAJOR 1
#define API_MINOR 3
#define API_PATCH 0


namespace azucar {
  namespace utils {

/*! \brief Version information for the azucar::utils API.
 *
 */
class Version
{
public:
	/*! \brief Return the major version number, e.g., 1 for "1.2.3"
	 *
	 */
	static int GetMajor();
	/*! \brief Return the minor version number, e.g., 2 for "1.2.3"
	 *
	 */
	static int GetMinor();
	/*! \brief Return the patch version number, e.g., 3 for "1.2.3"
	 *
	 */
	static int GetPatch();

	/*! \brief Return the full version number as a string, e.g., "1.2.3"
	 *
	 */
	static std::string GetVersion();

	/*! \brief Return true if the current version >= (major, minor, patch)
	 *
	 */
	static bool IsAtLeast(int major, int minor, int patch);

	/*! \brief Return true if the named feature is available in this version
	 *
	 */
	static bool HasFeature(const std::string &name);
};

  }
}

#endif
