/*! \file azucar/utils/xmlUtil.hpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#ifndef XMLUTIL_HPP_
#define XMLUTIL_HPP_

namespace xmlUtil {

XERCES_CPP_NAMESPACE_USE

// ---------------------------------------------------------------------------
//  This is a simple class that lets us do easy (though not terribly efficient)
//  trancoding of char* data to XMLCh data.
// ---------------------------------------------------------------------------
class XStr
{
public :
    // -----------------------------------------------------------------------
    //  Constructors and Destructor
    // -----------------------------------------------------------------------
    XStr(const char* const toTranscode)
    {
        // Call the private transcoding method
        fUnicodeForm = XMLString::transcode(toTranscode);
    }

    ~XStr()
    {
        XMLString::release(&fUnicodeForm);
    }


    // -----------------------------------------------------------------------
    //  Getter methods
    // -----------------------------------------------------------------------
    const XMLCh* unicodeForm() const
    {
        return fUnicodeForm;
    }

private :
    // -----------------------------------------------------------------------
    //  Private data members
    //
    //  fUnicodeForm
    //      This is the Unicode XMLCh format of the string.
    // -----------------------------------------------------------------------
    XMLCh*   fUnicodeForm;
};

} // end xmlUtil namespace

#define XMLSTR(str) xmlUtil::XStr(str).unicodeForm()


#endif /*XMLUTIL_HPP_*/
