/*! \file azucar/utils/database/mysqlppHelper.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include "azucar/utils/database/mysqlppHelper.hpp"

namespace azucar {
namespace utils {
namespace database {

  const std::string rowFieldAsString(const mysqlpp::Row &row, int fieldNum) {
#if (MYSQLPP_HEADER_VERSION >= MYSQLPP_VERSION(3, 0, 0))
	  std::string result;

	  row[fieldNum].to_string(result);

	  return result;
#else
	  return std::string(row.raw_data(fieldNum));
#endif
  }

  void mysqlpp_escape_string_correction(std::string &s) {
    if (!s.size()) {
      return;
    }

    for (unsigned int i = 0; i < s.size(); i++) {
      switch (s[i]) {
        case '\0':    // Must be escaped for "mysql"
          s[i] = '\\';
          i++;
          s.insert(i, "0", 1);
          break;
        case '\n':    // Must be escaped for logs
          s[i] = '\\';
          i++;
          s.insert(i, "n", 1);
          break;
        case '\r':
          s[i] = '\\';
          i++;
          s.insert(i, "r", 1);
          break;
        case '\\':
          s[i] = '\\';
          i++;
          s.insert(i, "\\", 1);
          break;
        case '\"':
          s[i] = '\\';
          i++;
          s.insert(i, "\"", 1);
          break;
        case '\'':    // Better safe than sorry
          s[i] = '\\';
          i++;
          s.insert(i, "\'", 1);
          break;
        case '\032':  // This gives problems on Win32
          s[i] = '\\';
          i++;
          s.insert(i, "Z", 1);
          break;
        default:
          break;
      }
    }
  }

} // end namespace
} // end namespace
} // end namespace
