/*! \file azucar/utils/deleterSync.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <boost/thread/xtime.hpp>

#include "azucar/utils/deleterSync.hpp"

using namespace std;

DeleterSync::DeleterSync() : deleterRunning_(false) {};
DeleterSync::~DeleterSync() {};

bool DeleterSync::waitDeleter(unsigned int secondsToWait) {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  if (deleterRunning_) {
    if (secondsToWait > 0) {
      boost::xtime timeLimit;
      if (xtime_get(&timeLimit, boost::TIME_UTC) != boost::TIME_UTC) {
        return false;
      } else {
        timeLimit.sec += secondsToWait; 
        return deleterDoneCondition_.timed_wait(scoped_lock, timeLimit);
      }
    } else {
      deleterDoneCondition_.wait(scoped_lock);
      return true;
    }
  } else {
    return true;
  }  
}

void DeleterSync::deleterRunning() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  deleterRunning_ = true;
}

void DeleterSync::deleterDone() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  deleterRunning_ = false;
  deleterDoneCondition_.notify_one();
}
