/*! \file azucar/utils/extensibleI.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <algorithm>
#include <functional>

#include "azucar/utils/extensibleI.hpp"

using namespace std;

void ExtensionMgr::registerProperty(PropertyObjectPtr property) {
  properties_.insert(property);
}

const set<PropertyObjectPtr>&
ExtensionMgr::getRegisteredProperties() const {
    return properties_;
}  

void ExtensibleI::registerExtensionMgr(ExtensionMgrPtr manager) {
  manager_ = manager;
  thisWeakPtr_ = shared_from_this();
}

ExtensibleI::~ExtensibleI() {
  if (manager_.get() != 0) {
    const set<PropertyObjectPtr> &properties
      = manager_->getRegisteredProperties();
    bind2nd(mem_fun(&PropertyObjectI::remove), thisWeakPtr_);      

    set<PropertyObjectPtr>::const_iterator it;
    for(it = properties.begin(); it != properties.end(); ++it) {
      (*it)->remove(thisWeakPtr_);
    }
  }
}  


