/*! \file azucar/utils/init.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <algorithm>

#include "azucar/utils/init.hpp"

using namespace std;

namespace azucar {
namespace utils {

Initializer::Initializer(
  const boost::function<void (Initializer&)> &initFunc,
  const boost::function<void (Initializer&)> &destroyFunc,
  const boost::filesystem::path &configFile) 
    : init_(initFunc), destroy_(destroyFunc), configFile_(configFile), 
      status(NON_INITIALIZED), dependents_()  {};

void Initializer::init() {
  init_(*this);
}

void Initializer::initDep(Initializer& dependent) {
  dependents_.push_back(&dependent);
  init();
}

void Initializer::destroy() {
  for_each(dependents_.begin(), dependents_.end(), 
           mem_fun(&Initializer::destroy));
  destroy_(*this);
}    

const boost::filesystem::path Initializer::getConfigFile() const {
  return configFile_;
}

void Initializer::setConfigFile(const boost::filesystem::path &configFile) {
  configFile_ = configFile;
}

}
}

