/*! \file azucar/utils/logger/util.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <string>

#include <boost/algorithm/string.hpp>

#include "azucar/utils/util.hpp"
#include "azucar/utils/logger/util.hpp"

using namespace std;

void createNewConfFile(const boost::filesystem::path &source,
    const boost::filesystem::path &target,
    const boost::filesystem::path &newLogFile) {

  azucar::utils::InputFile input(source);
  azucar::utils::OutputFile output(target);
  
  while(not azucar::utils::eof(input.stream)) {
    string line;
    getline(input.stream, line);
    
    boost::algorithm::trim(line);
    
    if (boost::algorithm::starts_with(line, "#")) {
      output.stream << line << endl;
    } else {
      // break line by = sign
      string::size_type equalPos = line.find('=');
      if (equalPos == string::npos) {
        output.stream << line << endl;
      } else {
      	// second parameter expresses length, so it must be equalPos to NOT INCLUDE = sign.
        string firstPart = line.substr(0, equalPos);
        boost::algorithm::trim(firstPart);
      
        // Does it have a last part ending in .fileName?
        if (boost::algorithm::ends_with(firstPart, ".fileName")) {
        // In such a case, replace X = Y with X = newLogFile
        	// second parameter to substr expresses length, so it must be equalPos to
        	// NOT INLUCE = sign.
          string newLine = line.substr(0, equalPos)
            + "=" + newLogFile.string();
          output.stream << newLine << endl;
        } else {
          output.stream << line << endl;
        }
      }
    }
  }
  
  output.close();
  input.close();

}
