/*! \file azucar/utils/mail/mailCommon.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <string>
#include <sstream>
#include <cassert>

#include <boost/lexical_cast.hpp>

#include <boost/filesystem/exception.hpp>
#include <boost/iostreams/filtering_stream.hpp>

#include <boost/preprocessor/iteration/local.hpp>
#include <boost/preprocessor/cat.hpp>
#include <boost/preprocessor/tuple.hpp>
#include <boost/preprocessor/seq.hpp>
#include <boost/preprocessor/stringize.hpp>
#include <boost/preprocessor/punctuation.hpp>

#include <loki/ScopeGuard.h>




#include "azucar/utils/util.hpp"
#include "azucar/utils/exceptions.hpp"
#include "azucar/utils/mail/mailCommon.hpp"
#include "azucar/utils/configurationReader.hpp"

using namespace std;
using namespace configuration;

const std::string MailConfiguration::category_name = "MailConfiguration";

log4cpp::Category &MailConfiguration::logger() {
  static log4cpp::Category &theLogger =   
    log4cpp::Category::getInstance(category_name);
  return theLogger;
}

#define MYMACRO(data) BOOST_PP_TUPLE_ELEM(2,1,data) \
MailConfiguration::BOOST_PP_CAT(BOOST_PP_TUPLE_ELEM(2,0,data), _);
#define BOOST_PP_LOCAL_MACRO(n) MYMACRO(BOOST_PP_SEQ_ELEM(n, MAILCOMMON_var_seq))
#define BOOST_PP_LOCAL_LIMITS (0, BOOST_PP_SEQ_SIZE(MAILCOMMON_var_seq)-1)
#include BOOST_PP_LOCAL_ITERATE()
#undef MYMACRO


#define MYMACRO(data) const BOOST_PP_TUPLE_ELEM(2,1,data) \
 &MailConfiguration::BOOST_PP_TUPLE_ELEM(2,0,data) \
 = MailConfiguration::BOOST_PP_CAT(BOOST_PP_TUPLE_ELEM(2,0,data),_);
#define BOOST_PP_LOCAL_MACRO(n) MYMACRO(BOOST_PP_SEQ_ELEM(n, MAILCOMMON_var_seq))
#define BOOST_PP_LOCAL_LIMITS (0, BOOST_PP_SEQ_SIZE(MAILCOMMON_var_seq)-1)
#include BOOST_PP_LOCAL_ITERATE()
#undef MYMACRO

void MailConfiguration::read(const boost::filesystem::path &configFile) {
  ConfigurationReader reader;
  
  try {
    reader.init(configFile);
  } catch (azucar::utils::BaseException &ex) {
    throw;
  }

  
#define MYMACRO(data) if BOOST_PP_LPAREN() not reader.existsTextOption \
BOOST_PP_LPAREN() BOOST_PP_STRINGIZE(BOOST_PP_TUPLE_ELEM(2,0,data)) \
BOOST_PP_RPAREN() BOOST_PP_RPAREN() { throw azucar::utils::BaseException \
  BOOST_PP_LPAREN() std::string("Missing option") + \
  BOOST_PP_STRINGIZE(BOOST_PP_TUPLE_ELEM(2,0,data)) BOOST_PP_RPAREN() \
  ; } 
#define BOOST_PP_LOCAL_MACRO(n) MYMACRO(BOOST_PP_SEQ_ELEM(n, MAILCOMMON_var_seq))
#define BOOST_PP_LOCAL_LIMITS (0, BOOST_PP_SEQ_SIZE(MAILCOMMON_var_seq)-1)
#include BOOST_PP_LOCAL_ITERATE()
#undef MYMACRO


  try {
#define MYMACRO(data) BOOST_PP_CAT(BOOST_PP_TUPLE_ELEM(2,0,data),_) \
 = readOption<BOOST_PP_TUPLE_ELEM(2,1,data)> \
 BOOST_PP_LPAREN() reader BOOST_PP_COMMA() \
 BOOST_PP_STRINGIZE(BOOST_PP_TUPLE_ELEM(2,0,data)) BOOST_PP_RPAREN();
#define BOOST_PP_LOCAL_MACRO(n) MYMACRO(BOOST_PP_SEQ_ELEM(n, MAILCOMMON_var_seq))
#define BOOST_PP_LOCAL_LIMITS (0, BOOST_PP_SEQ_SIZE(MAILCOMMON_var_seq)-1)
#include BOOST_PP_LOCAL_ITERATE()
#undef MYMACRO
  } catch (...) {
    // ?? how to detect conversion problems
    throw azucar::utils::BaseException("Problems reading options");
  }

  reader.release();  
}

