/*! \file azucar/utils/mail/mailInit.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <vmime/vmime.hpp>
#include <vmime/platforms/posix/posixHandler.hpp>

#include "azucar/utils/mail/mailInit.hpp"
#include "azucar/utils/mail/mailCommon.hpp"
#include "azucar/utils/xmlInit.hpp"

using namespace std;
using azucar::utils::Initializer;

namespace {
  /*! \brief Initialize VMIME mail library.
   * 
   * If state is Initializer::NON_INITIALIZED, initialize
   * library and new state is Initializer::INITIALIZED.
   * Otherwise, do nothing.
   */
  void init(Initializer& initObj) {
    if (initObj.status == Initializer::NON_INITIALIZED) {
      xmlInitializer.initDep(initObj);
      MailConfiguration::read(initObj.getConfigFile());
      vmime::platform::setHandler<vmime::platforms::posix::posixHandler>();
      initObj.status = Initializer::INITIALIZED;
    }
  }

  /*! \brief Finalize VMIME mail library.
   * 
   * If state is Initializer::INITIALIZED, finalize the library
   * and set state to Initializer::DESTROYED.
   * Otherwise, do nothing.
   */
  void destroy(Initializer& initObj) {
    // do nothing
    if (initObj.status == Initializer::INITIALIZED) {
      initObj.status = Initializer::DESTROYED;
    }
  }
  
} // end anonymous namespace

Initializer mailInitializer(&init, &destroy);
