/*! \file azucar/utils/mail/mailUtil.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include "azucar/utils/mail/mailUtil.hpp"
#include "azucar/utils/mail/mailCommon.hpp"
#include "azucar/utils/util.hpp"

using namespace std;

interactiveAuthenticator::interactiveAuthenticator() 
  : m_username(), m_password() {};

interactiveAuthenticator::interactiveAuthenticator(
    const std::string &user, const std::string &key)
    : m_username(user), m_password(key) {}; 

const std::vector <vmime::ref <vmime::security::sasl::SASLMechanism> > 
interactiveAuthenticator::getAcceptableMechanisms
    (const std::vector <vmime::ref <vmime::security::sasl::SASLMechanism> >& available,
     vmime::ref <vmime::security::sasl::SASLMechanism> suggested) const
  {
    return defaultSASLAuthenticator::getAcceptableMechanisms(available, suggested);
  }

void 
interactiveAuthenticator::setSASLMechanism(vmime::ref <vmime::security::sasl::SASLMechanism> mech)
  {
    defaultSASLAuthenticator::setSASLMechanism(mech);
  }

const vmime::string 
interactiveAuthenticator::getUsername() const
  {
    if (m_username.empty())
      m_username = MailConfiguration::mail_sasl_user;

    return m_username;
  }

const vmime::string 
interactiveAuthenticator::getPassword() const
  {
    if (m_password.empty())
      m_password = MailConfiguration::mail_sasl_passwd;

    return m_password;
  }

void MailSender::sendMail(vmime::ref<vmime::message> msg) {
    boost::mutex::scoped_lock scoped_lock(m_mutex);
    vmime::ref<vmime::net::session> sess = vmime::create<vmime::net::session>();
    vmime::utility::url url("smtp",
      MailConfiguration::smtp_server_url);
    vmime::ref<vmime::net::transport> tr;
    if (MailConfiguration::use_smtp_auth) {
      tr = sess->getTransport(url, vmime::create <interactiveAuthenticator>());
      tr->setProperty("options.need-authentication", true);
    } else {
      tr = sess->getTransport(url);
      tr->setProperty("options.need-authentication", false);
    }
    tr->connect();
    // since LOKI_ON_BLOCK_EXIT will call the function before
    // tr is destroyed, we make sure we can do *tr.
    ON_BLOCK_EXIT(&vmime::net::transport::disconnect, *tr);
    tr->send(msg, 0);
}

void MailSender::sendMail(vmime::ref<vmime::message> msg, 
      const string &smtp_user, const string &smtp_key) {
    vmime::ref<vmime::net::session> sess = vmime::create<vmime::net::session>();
    vmime::utility::url url("smtp",
      MailConfiguration::smtp_server_url);

    vmime::ref<vmime::net::transport> tr 
      = sess->getTransport(url, vmime::create <interactiveAuthenticator>(smtp_user, smtp_key));
    tr->setProperty("options.need-authentication", true);
    tr->connect();
    // since LOKI_ON_BLOCK_EXIT will call the function before
    // tr is destroyed, we make sure we can do *tr.
    ON_BLOCK_EXIT(&vmime::net::transport::disconnect, *tr);
    tr->send(msg, 0);
}
