/*! \file azucar/utils/network.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <boost/thread/thread.hpp>



#include "azucar/utils/network.hpp"

using namespace std;

using boost::asio::ip::tcp;

namespace network {

log4cpp::Category &logger() {
  static log4cpp::Category &theLogger =   
    log4cpp::Category::getInstance(category_name);
  return theLogger;
}


NetworkException::NetworkException(const std::string &msg) : azucar::utils::BaseException(msg) {};

NetworkException::~NetworkException() throw () {};

NetAsyncService::NetAsyncService(boost::asio::io_service& service) 
    : service_(service), terminateSet_(false) {};
    
bool NetAsyncService::isTerminateSet() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  return terminateSet_;      
}

void NetAsyncService::asyncTerminate() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  terminateSet_ = true;  
}

void NetAsyncService::blockingRun() {
  while(not isTerminateSet()) {
    service_.run();
    // if no service request is active, run will finish
    // and the service must be reset, and yield the
    // processor to another thread, befor calling run again.
    //
    // It must be checked whether there is any 
    // problem in between call of run() and reset(), in case
    // a client posts a network request. 
    
    service_.reset();
    boost::thread::yield();
  }
  
}

} // end namespace network
