/*! \file azucar/utils/processInit.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <signal.h>
//#include <iostream>

#include "azucar/utils/processInit.hpp"
#include "azucar/utils/exceptions.hpp"

using namespace std;
using azucar::utils::Initializer;

namespace { // anonymous namespace

  struct sigaction originalAction;
  
  void ignoreChildren(int signo) {
//    cerr << "SIGCHLD received." << endl;
    return;
  }
  
  
  void init(Initializer& initObj) {
    if ((initObj.status == Initializer::NON_INITIALIZED) 
        or (initObj.status == Initializer::DESTROYED)) {
      
      struct sigaction act;
      act.sa_handler = ignoreChildren;
      sigemptyset(&act.sa_mask);
      act.sa_flags = 0;
      
      if (sigaction(SIGCHLD, &act, &originalAction) < 0) {
        throw azucar::utils::BaseException("Could not set sigaction");
      }
      
      initObj.status = Initializer::INITIALIZED;
    }
  }
  
  /*! \brief Finalize Xerces XML Module.
   * 
   * If state of library is Initializer::INITIALIZED,
   * finalize process library.
   * New state is Initializer::DESTROYED.
   */
  void destroy(Initializer& initObj) {
    if (initObj.status == Initializer::INITIALIZED) {
      struct sigaction tmpAct;
      if (sigaction(SIGCHLD, &originalAction, &tmpAct) < 0) {
        throw azucar::utils::BaseException("Could not set sigaction");
      }
      initObj.status = Initializer::DESTROYED;
    }
  }
  
};

Initializer processInitializer(&init, &destroy);
