/*! \file azucar/utils/threadMonitor.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include "azucar/utils/threadMonitor.hpp"

using namespace std;

void ThreadMonitor::waitNotification() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  if (immediateExit_ or (normalThreadsRunning_ == 0)) {
    // check if the condition is already happening
    return;
  } else {
    // wait a little, to find out if all threads have
    // actually started normal work
    
    // recheck normalThreadsToWait_ == 0
    
    // now, wait
    notifyCondition.wait(scoped_lock);
  }
}

void ThreadMonitor::notifyNormalThreadEnd() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  normalThreadsRunning_--;
  if (normalThreadsRunning_ == 0) {
    notifyCondition.notify_one();
  }
}

void ThreadMonitor::addNormalThread() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  normalThreadsRunning_++;
}

void ThreadMonitor::notifyErrorThreadEnd() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  immediateExit_ = true;
  // if nobody is waiting, the notify_one has no effect
  // that is why immediateExit_ is set.
  notifyCondition.notify_one();
}

void ThreadMonitor::notifyKeyThreadEnd() {
  boost::mutex::scoped_lock scoped_lock(m_mutex);
  immediateExit_ = true;
  // if nobody is waiting, the notify_one has no effect
  // that is why immediateExit_ is set.
  notifyCondition.notify_one();
}

ThreadMonitor::~ThreadMonitor() {};


