/*! \file azucar/utils/zip/zipI.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/convenience.hpp>

#include <log4cpp/Category.hh>
#include <log4cpp/PropertyConfigurator.hh>
#include <log4cpp/NDC.hh>


#include "azucar/utils/zip/zipI.hpp"
#include "azucar/utils/zip/zipCommon.hpp"
#include "azucar/utils/util.hpp"
#include "azucar/utils/process.hpp"

using namespace std;

namespace {
const std::string category_name = "zipI_cat";

log4cpp::Category &logger() {
  static log4cpp::Category &theLogger =   
    log4cpp::Category::getInstance(category_name);
  return theLogger;
}
}


namespace zip {

void zip(const boost::filesystem::path &inputFilePath,
              const boost::filesystem::path &outputFilePath,
              const boost::filesystem::path &workingDirectory) {

      vector<string> argVector; // empty, except for the scriptName
      argVector.push_back(ZipConfiguration::zip_executable_path.string()); // make sure to pass only the ending
      argVector.push_back(string("-r"));
      argVector.push_back(outputFilePath.string());
      argVector.push_back(string("."));
      argVector.push_back(string("-i@") + inputFilePath.string());
      std::map<string,string> envVector = convertEnvironToMap(environ);

      /*int returnStatus =*/ processExecute(ZipConfiguration::zip_executable_path.string(),
              argVector, envVector, ZipConfiguration::null_device_path.string(),
              ZipConfiguration::null_device_path.string(),
              workingDirectory.string());
              

}

void unzip(const boost::filesystem::path &inputFilePath,
              const boost::filesystem::path &workingDirectory) {

      vector<string> argVector; // empty, except for the scriptName
      argVector.push_back(ZipConfiguration::unzip_executable_path.string()); // make sure to pass only the ending
      argVector.push_back(inputFilePath.string());
      std::map<string,string> envVector = convertEnvironToMap(environ);

      /*int returnStatus =*/ processExecute(ZipConfiguration::unzip_executable_path.string(),
              argVector, envVector, ZipConfiguration::null_device_path.string(),
              ZipConfiguration::null_device_path.string(),
              workingDirectory.string());
}

void unzipList(const boost::filesystem::path &inputFilePath,
              const boost::filesystem::path &listFilePath,
              const boost::filesystem::path &workingDirectory) {

      vector<string> argVector; // empty, except for the scriptName
      argVector.push_back(ZipConfiguration::unzip_executable_path.string()); // make sure to pass only the ending
      argVector.push_back("-Z");
      argVector.push_back("-1");
      argVector.push_back(inputFilePath.string());
      std::map<string,string> envVector = convertEnvironToMap(environ);

      /*int returnStatus =*/ processExecute(ZipConfiguration::unzip_executable_path.string(),
              argVector, envVector, ZipConfiguration::null_device_path.string(),
              listFilePath.string(),
              workingDirectory.string());
}

void unzipFile(const boost::filesystem::path &inputFilePath,
              const boost::filesystem::path &fileToExtract,
              const boost::filesystem::path &workingDirectory) {

      vector<string> argVector; // empty, except for the scriptName
      argVector.push_back(ZipConfiguration::unzip_executable_path.string()); // make sure to pass only the ending
      argVector.push_back(inputFilePath.string());
      argVector.push_back(fileToExtract.string());
      std::map<string,string> envVector = convertEnvironToMap(environ);

      /*int returnStatus =*/ processExecute(ZipConfiguration::unzip_executable_path.string(),
              argVector, envVector, ZipConfiguration::null_device_path.string(),
              ZipConfiguration::null_device_path.string(),
              workingDirectory.string());
}

} // end namespace zip
