/*! \file azucar/utils/zip/zipInit.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include "azucar/utils/zip/zipInit.hpp"
#include "azucar/utils/zip/zipCommon.hpp"
#include "azucar/utils/xmlInit.hpp"

using azucar::utils::Initializer;

namespace {
  /*! \brief Initialize Progeom module.
   * 
   * \exception azucar::utils::BaseXMLException If an error happens during
   *  initialization.
   */
  void init(Initializer& initObj) {
    if (initObj.status == Initializer::NON_INITIALIZED) {
      xmlInitializer.initDep(initObj);
      ZipConfiguration::read(initObj.getConfigFile());
      initObj.status = Initializer::INITIALIZED;
    }
  }
  
  void destroy(Initializer& initObj) {
    if (initObj.status == Initializer::INITIALIZED) {
      initObj.status = Initializer::DESTROYED;
    }
  }
}; // end anonymous namespace

Initializer zipInitializer(&init, &destroy);
