/*! \file test/apps/test_CGILint.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <cassert>
#include <string>
#include <fstream>
#include <vector>

#include <boost/test/unit_test.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/convenience.hpp>

#include "azucar/utils/process.hpp"
#include "azucar/utils/util.hpp"
#include "azucar/utils/testUtils.hpp"
#include "test/azucar/utils/testCommon.hpp"

using boost::unit_test::test_suite;
using boost::unit_test::test_case;
using namespace std;

BOOST_AUTO_TEST_SUITE(test_cgi)

  const string cgiLint_config_file = "scratch/CGILint_test.cfg";
  const string cgiLint_data_file = "scratch/CGILint_test.dat";
  const string echo_string = "This is the echo string";
  const string result_pattern_file = "data/test/test_CGILint/test_cgi_output.txt";




  void callCGILint() {
    
      
    // use exec
    vector<string> argVector; // empty, except for the scriptName
    boost::filesystem::path cgiLintCompletePath
      = boost::filesystem::system_complete(test::settings.cgiLint_path);
      
    boost::filesystem::path currentPath
      = boost::filesystem::system_complete("scratch");
    argVector.push_back(cgiLintCompletePath.string()); // make sure to pass only the ending
    argVector.push_back(boost::filesystem::system_complete(cgiLint_config_file).string());
    argVector.push_back(boost::filesystem::system_complete(cgiLint_data_file).string());

    // environ should work, so there should be no exception
    // no thread access environ on write mode, so 
    // all threads can access it on read mode.
    std::map<string,string> envVector = convertEnvironToMap(environ);

    /*int returnStatus =*/ processExecute(cgiLintCompletePath.string(),
              argVector, envVector, string("/dev/null"),
              string("/dev/null"),
              currentPath.string());
              
    // we will check the program has worked in future steps.              

  }

  BOOST_AUTO_TEST_CASE(test_simple) {

    // create CGILint_test.cfg

    if (boost::filesystem::exists(cgiLint_config_file)) {
      boost::filesystem::remove(cgiLint_config_file);
    }
    

    ofstream configFile(cgiLint_config_file.c_str());
    configFile << "REQUEST_METHOD = post" << endl
      << "SERVER_ROOT = ." << endl
      << "SCRIPT_NAME = " 
      << boost::filesystem::system_complete(test::settings.testCGILint_path).string()
      << endl
      << "LD_LIBRARY_PATH = /usr/local/cgicc/lib:../build/azucar/utils"
      << endl;
    configFile.close();
    
    // create CGILint_test.dat

    if (boost::filesystem::exists(cgiLint_data_file)) {
      boost::filesystem::remove(cgiLint_data_file);
    }
    
    ofstream dataFile(cgiLint_data_file.c_str());
    dataFile << "echostring = " << echo_string << endl;
    dataFile.close();
    
    // launch CGILint
    callCGILint();    

    // read CGIResult
    // check the reply is good
      BOOST_CHECK(test::compareFiles(result_pattern_file, test::settings.cgi_output_file));
  }
// end test_cgi
BOOST_AUTO_TEST_SUITE_END()
