/*! \file test/apps/test_CGILint_server.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <string>
#include <iostream>

#include "cgicc/CgiDefs.h"
#include "cgicc/Cgicc.h"
#include "cgicc/HTTPHTMLHeader.h"
#include "cgicc/HTMLClasses.h"

#include <log4cpp/Category.hh>
#include <log4cpp/PropertyConfigurator.hh>


#include "azucar/utils/exceptions.hpp"

using namespace std;
using namespace cgicc;

const std::string category_name = "main_cat";

log4cpp::Category &logger() {
  static log4cpp::Category &theLogger =   
    log4cpp::Category::getInstance(category_name);
  return theLogger;
}


/*! \brief Error communication via HTTP/HTML when no logging
 * services are available.
 * 
 * When logging is available, a more customized and rich
 * HTML output should be used.
 * 
 * However, this function can be used even with no logging
 * services available.
 */
void reply(const string &msg) {
    HTTPHTMLHeader().render(cout);
    cout << HTMLDoctype(HTMLDoctype::eStrict) << endl;

  // read request status output
  
  string htmlText = "<body>";
  htmlText += msg;
  htmlText += "</body>";

  cout << htmlText;
  
  cout << endl;
    
}

void normalReply(const string &msg) {
  reply(msg);
}

void replyWithSimpleErrorMessage(const string &msg) {
  reply(msg);
}

int server() {

  try {
    // Create a new Cgicc object containing all the CGI data
    Cgicc cgi;
    
    // if shouldBreak is true, no further processing must be done
    bool shouldBreak = false;
    string echoString;

    // Get a pointer to the environment
//    const CgiEnvironment& env = cgi.getEnvironment();
    const_form_iterator iter;
    iter = cgi.getElement("echostring");
    if (iter != cgi.getElements().end()) {
      echoString = iter->getValue();
    } else {
      replyWithSimpleErrorMessage("Error: No echo string specified.");
      shouldBreak = true;
    }
    
    
    if (not shouldBreak) {
      normalReply(echoString);
    }    
  } catch (azucar::utils::BaseException &ex) {
    logger().errorStream() << ex.getMessage() 
      << log4cpp::CategoryStream::EOL;
    replyWithSimpleErrorMessage("Technical problem.");
    return 1;        
  } catch (exception &e) {
    logger().errorStream() << e.what() 
      << log4cpp::CategoryStream::EOL;
    replyWithSimpleErrorMessage("Technical problem.");
    return 1;
  }

  return 0;
}

int main(int argc, char **argv) {
  return server();
}
