/*! \file test/azucar/utils/testCommon.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include "test/azucar/utils/testCommon.hpp"

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <string>
#include <set>
#include <exception>
#include <iostream>

using namespace std;
using namespace boost::property_tree::xml_parser;

namespace test {

Settings::Settings() : testIPCProcessClient_path() {};

void Settings::load(const std::string &filename) {
    // Create empty property tree object
    using boost::property_tree::ptree;
    ptree pt;

    // Load XML file and put its contents in property tree.
    // No namespace qualification is needed, because of Koenig
    // lookup on the second argument. If reading fails, exception
    // is thrown.
    read_xml(filename, pt, trim_whitespace);

	  testIPCProcessClient_path = pt.get<std::string>("props.testIPCProcessClient_path");
    imap_user = pt.get<std::string>("props.imap_user");
	  imap_key = pt.get<std::string>("props.imap_key");
	  imap_url = pt.get<std::string>("props.imap_url");
	  imap_email = pt.get<std::string>("props.imap_email");

	  zip_executable_path = pt.get<std::string>("props.zip_executable_path");
	  unzip_executable_path = pt.get<std::string>("props.unzip_executable_path");
	  null_device_path = pt.get<std::string>("props.null_device_path");
	  smtp_server_url = pt.get<std::string>("props.smtp_server_url");
	  use_smtp_auth = pt.get<bool>("props.use_smtp_auth");
	  mail_expeditor = pt.get<std::string>("props.mail_expeditor");
	  mail_sasl_user = pt.get<std::string>("props.mail_sasl_user");
	  mail_sasl_passwd = pt.get<std::string>("props.mail_sasl_passwd");
	  cgiLint_path = pt.get<std::string>("props.cgiLint_path");
	  testCGILint_path = pt.get<std::string>("props.testCGILint_path");
	  cgi_output_file = pt.get<std::string>("props.cgi_output_file");

}

void Settings::save(const std::string &filename) {
    // Create empty property tree object
    using boost::property_tree::ptree;
    ptree pt;

    // Put log filename in property tree
    pt.put("props.testIPCProcessClient_path", testIPCProcessClient_path);
    pt.put("props.imap_user", imap_user);
    pt.put("props.imap_key", imap_key);
    pt.put("props.imap_url", imap_url);
    pt.put("props.imap_email", imap_email);
    pt.put("props.zip_executable_path", zip_executable_path);
    pt.put("props.unzip_executable_path", unzip_executable_path);
    pt.put("props.null_device_path", null_device_path);
    pt.put("props.smtp_server_url", smtp_server_url);
    pt.put("props.use_smtp_auth", use_smtp_auth);
    pt.put("props.mail_expeditor", mail_expeditor);
    pt.put("props.mail_sasl_user", mail_sasl_user);
    pt.put("props.mail_sasl_passwd", mail_sasl_passwd);
    pt.put("props.cgiLint_path", cgiLint_path);
    pt.put("props.testCGILint_path", testCGILint_path);
    pt.put("props.cgi_output_file", cgi_output_file);

    // Write property tree to XML file
    write_xml(filename, pt, std::locale(),
	      xml_writer_settings<char>(char(' '), 2, widen<char>("utf-8")));
}
} // end namespace
