/*! \file test/azucar/utils/test_configurationReader.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <cassert>
#include <string>
#include <fstream>

#include <boost/test/unit_test.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>

#include "azucar/utils/configurationReader.hpp"

using boost::unit_test::test_suite;
using boost::unit_test::test_case;
using namespace std;
using namespace configuration;

BOOST_AUTO_TEST_SUITE(test_configurationReader)

	const string config_filename = "scratch/test_config_reader.xml";
  const string option1 = "option1";
  const string option2 = "option2";
  const string option3 = "option3";
  const string option3_value = "option3_value";
  
  BOOST_AUTO_TEST_CASE(simple_test) {
    ofstream outputConfig(config_filename.c_str());
    
    outputConfig << "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" << endl;
    outputConfig << "<configuration>" << endl;
    outputConfig << "<option1>nada</option1>" << endl;
    outputConfig << "<option3>option3_value</option3>" << endl;
    outputConfig << "</configuration>" << endl;
    
    outputConfig.close();
    ConfigurationReader reader;
    reader.init(config_filename);
    BOOST_CHECK(not reader.existsTextOption(option2));
    BOOST_CHECK(reader.existsTextOption(option1));
    BOOST_REQUIRE(reader.existsTextOption(option3));
    BOOST_CHECK_EQUAL(reader.getTextOption(option3), option3_value);
    
    boost::filesystem::remove(config_filename);    
  }

BOOST_AUTO_TEST_SUITE_END()
