/*! \file test/azucar/utils/test_monitoredQueue.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <boost/thread/thread.hpp>
#include <boost/thread/xtime.hpp>
#include <boost/bind.hpp>
#include <boost/ref.hpp>

#include "azucar/utils/monitoredQueue.hpp"
#include "azucar/utils/util.hpp"

#include "test/azucar/utils/test_monitoredQueue.hpp"


namespace {

using boost::unit_test::test_suite;
using boost::unit_test::test_case;
using namespace std;

const int num_elements = 3;

void runProducer(MonitoredQueue<double> &queue,
                 double elements[num_elements]) {

  for(int i=0; i < num_elements; ++i) {
    azucar::utils::threadSleep(1);
    
    queue.pushBack(elements[i]);
  }                    
}

void runConsumer(MonitoredQueue<double> &queue,
        double results[num_elements]) {
  for(int i=0; i < num_elements; ++i) {
    double element = queue.popFront();
    results[i] = element;
  }
}

void test_producer_consumer() {
  MonitoredQueue<double> queue;
  double elementsToWork[num_elements] = {1.2, 3.2, 4.3};
  double results[num_elements];
  
  // create producer
  // create consumer

  boost::thread consumer(boost::bind(runConsumer, boost::ref(queue),
    results));
  boost::thread producer(boost::bind(runProducer, boost::ref(queue),
    elementsToWork));
  
  // wait for complation
  producer.join();
  consumer.join();
  
  BOOST_CHECK_EQUAL_COLLECTIONS(elementsToWork, elementsToWork+3,
      results, results+3);
      
  queue.pushBack(5.0);
  double result;
  BOOST_REQUIRE(queue.popFrontDoNotWait(result));
  BOOST_REQUIRE_EQUAL(result, 5.0);
  BOOST_REQUIRE(not queue.popFrontDoNotWait(result));      
  
}

} // end namespace

boost::unit_test::test_suite *test_monitoredQueue_init() {
    test_suite* test= BOOST_TEST_SUITE(test_monitoredQueue);

    // this example will pass cause we know ahead of time number of expected failures
    test_case *simpleTest = BOOST_TEST_CASE( &test_producer_consumer );
    test->add(simpleTest, 0, 4);
    
    return test;
}
