/*! \file test/azucar/utils/test_processClient.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <cassert>
#include <vector>
#include <string>

#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/shared_array.hpp>
#include <boost/algorithm/string.hpp>

#include "azucar/utils/process.hpp"
#include "azucar/utils/util.hpp"

using namespace std;


void simpleMultiplier(void *input, size_t inputSize,
        void *output, size_t outputSize) {
  
  double *inputDouble = static_cast<double*>(input);
  
  double *outputDouble = static_cast<double*>(output);
  
  int numDoubles = outputSize / sizeof(double);
  
  for(int i=0; i < numDoubles; ++i) {
    outputDouble[i] = inputDouble[i] * 2;
  }
  
}

class Executer {
public:
  boost::shared_array<char> args_;
  size_t argsSize_;
  boost::shared_array<char> results_;
  size_t resultsSize_;
public: 
  Executer(size_t argsSize, size_t resultsSize)
    : args_(new char[argsSize]), argsSize_(argsSize),
      results_(new char[resultsSize]), resultsSize_(resultsSize) {}; 

  void operator()(void) {
    simpleMultiplier(args_.get(), argsSize_, 
      results_.get(), resultsSize_);
  }
};


int main(int argc, char **argv) {
  // first arg -> shmid
  // second arg -> semid
  // third arg -> input size
  // fourth arg -> output size

  if (argc == 5) {
    int shmid = boost::lexical_cast<int>(string(argv[1]));
    int semid = boost::lexical_cast<int>(string(argv[2]));
    size_t argsSize = boost::lexical_cast<size_t>(string(argv[3]));
    size_t resultsSize = boost::lexical_cast<size_t>(string(argv[4]));

    Executer testExecuter(argsSize, resultsSize);

    clientExecuteIPC(shmid, semid, testExecuter.args_.get(), argsSize,
            testExecuter.results_.get(), resultsSize, 2, 
            testExecuter);
  } else {
    try {
      std::string inputFileName(argv[1]);
      std::string outputFileName(argv[2]);
      int numLines = boost::lexical_cast<int>(string(argv[3]));
  
      boost::shared_array<double> operandOne(new double[numLines]);
      boost::shared_array<double> operandTwo(new double[numLines]);
      boost::shared_array<double> results(new double[numLines]);    
      
      azucar::utils::InputFile inputData(inputFileName);
      double *opOnePtr = operandOne.get();
      double *opTwoPtr = operandTwo.get();
  
      for(int i = 0; i < numLines; ++i) {
        if (azucar::utils::eof(inputData.stream)) {
          // error
          throw azucar::utils::BaseException("Wrong input file format.");
        }
        string line;
        getline(inputData.stream, line);
        boost::algorithm::trim(line);
        std::vector<std::string> tokens;
        boost::algorithm::split(tokens, line, 
          boost::algorithm::is_any_of(","));
        // read from file
        // break into components
        vector<string>::iterator it;
        for(it = tokens.begin(); it != tokens.end(); ++it) {
          boost::algorithm::trim(*it);
        }
        assert(tokens.size() == 2);
        
        *opOnePtr = boost::lexical_cast<double>(tokens[0]);
        opOnePtr++;
        *opTwoPtr = boost::lexical_cast<double>(tokens[1]);
        opTwoPtr++;
        
        // assign
        // incr pointers
      }
      
      if (not azucar::utils::eof(inputData.stream)) {
        throw azucar::utils::BaseException("Wrong input file format (too much data).");
      }
  
      inputData.close();

      for(int i = 0; i < numLines; ++i) {
        results[i] = operandOne[i] + operandTwo[i];
      }
  
      azucar::utils::OutputFile outputData(outputFileName);
      double *resultsPtr = results.get();
      for(int i = 0; i < numLines; ++i, resultsPtr++) {
        outputData.stream << (*resultsPtr) << "\n";
      }
      
      outputData.close();
    } catch (...) {
      return 1; // ERROR!!!
    }
    
    
  }
 
  return 0; 
}
