/*! \file test/azucar/utils/test_propertyDB.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <iostream>
#include <string>

#include <boost/test/unit_test.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/python.hpp>
#include <boost/format.hpp>

#include "azucar/utils/propertyDB.hpp"
#include "azucar/utils/propertyDBMap.hpp"

using namespace std;

BOOST_AUTO_TEST_SUITE(global)

BOOST_AUTO_TEST_SUITE(test_azucar_utilities_propertyDBMap)

BOOST_AUTO_TEST_CASE(testCreation) {
	// create PropertyDBMap
	boost::shared_ptr<azucar::utils::PropertyDB> propdb(new azucar::utils::PropertyDBMap());
}

BOOST_AUTO_TEST_CASE(testPropertySetGet) {
	boost::shared_ptr<azucar::utils::PropertyDB> propdb(new azucar::utils::PropertyDBMap());

	const std::string prop_name = "myproperty";
	const std::string prop_value = "myvalue";

	propdb->set(prop_name, prop_value);

	BOOST_CHECK(propdb->has(prop_name));

	BOOST_CHECK_EQUAL(propdb->get(prop_name), prop_value);
}

BOOST_AUTO_TEST_CASE(testCopy) {
	boost::shared_ptr<azucar::utils::PropertyDB> propdb(new azucar::utils::PropertyDBMap());

	const std::string prop_name = "myproperty";
	const std::string prop_value = "myvalue";

	propdb->set(prop_name, prop_value);

	boost::shared_ptr<azucar::utils::PropertyDB> propdbcopy = propdb->clone();

	BOOST_CHECK(propdbcopy->has(prop_name));

	BOOST_CHECK_EQUAL(propdbcopy->get(prop_name), prop_value);

}

// end test_azucar_utilities_propertyDBMap
BOOST_AUTO_TEST_SUITE_END()

// end global
BOOST_AUTO_TEST_SUITE_END()
