/*! \file test/azucar/utils/zip/test_zip.cpp
 *
 * \author Jorge Estrada
 *
 * Copyright 2013-2014 Jorge Estrada.
 *
 * This file is part of the azucar.cppUtils project.
 * It is subject to the license terms in the
 * LICENSE file found in the top-level directory of this distribution.
 * No part of the azucar.cppUtils project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms contained
 * in the LICENSE file.
 */

#include <cassert>
#include <string>
#include <fstream>
#include <vector>

#include <boost/test/unit_test.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/convenience.hpp>

#include "azucar/utils/zip/zipI.hpp"
#include "azucar/utils/util.hpp"
#include "azucar/utils/testUtils.hpp"

using boost::unit_test::test_suite;
using boost::unit_test::test_case;
using namespace std;

BOOST_AUTO_TEST_SUITE(test_zip)

  const string pdb1_path = "data/test/common/problem1.pdb";
  const string pdb2_path = "data/test/common/problem2.pdb";
  const string temp_out_path = "scratch/temp.chk.zip";
  const string temp_list_path = "scratch/temp.zip.lst";
  const string unzipped_pdb1_path = "scratch/data/test/common/problem1.pdb";
  const string unzipped_pdb2_path = "scratch/data/test/common/problem2.pdb";
  const string working_path = ".";
  const string extracting_working_path = "scratch";




  BOOST_AUTO_TEST_CASE(test_one) {

    if (boost::filesystem::exists(temp_list_path)) {
      boost::filesystem::remove(temp_list_path);
    }
    if (boost::filesystem::exists(temp_out_path)) {
      boost::filesystem::remove(temp_out_path);
    }
    if (boost::filesystem::exists(unzipped_pdb1_path)) {
      boost::filesystem::remove(unzipped_pdb1_path);
    }
    if (boost::filesystem::exists(unzipped_pdb2_path)) {
      boost::filesystem::remove(unzipped_pdb2_path);
    }
    
    // create list
    azucar::utils::OutputFile outputFileHandle(temp_list_path);
    outputFileHandle.stream << pdb1_path << endl; 
    outputFileHandle.stream << pdb2_path << endl;
    outputFileHandle.close(); 
    
    zip::zip(temp_list_path, temp_out_path, working_path);
    zip::unzip(boost::filesystem::complete(temp_out_path), 
      extracting_working_path);
    
    BOOST_CHECK(test::compareFiles(pdb1_path, unzipped_pdb1_path));
    BOOST_CHECK(test::compareFiles(pdb2_path, unzipped_pdb2_path));
  }

// end zip
BOOST_AUTO_TEST_SUITE_END()
